# -*- coding: utf-8 -*-
# Copyright (C) 2023 mhertz <yourname@example.com>
#
# Basic plugin template created by the Deluge Team.
#
# This file is part of wslimit and is licensed under GNU GPL 3.0, or later,
# with the additional special exception to link portions of this program with
# the OpenSSL library. See LICENSE for more details.
from __future__ import unicode_literals

import logging

import deluge.configmanager
from deluge.core.rpcserver import export
from deluge.plugins.pluginbase import CorePluginBase
import deluge.component as component
from twisted.internet import reactor
import os
import ctypes
import threading
import platform

log = logging.getLogger(__name__)

DEFAULT_PREFS = {
    'mem': 512,
    'prio': 'below-normal'
}

class Core(CorePluginBase):
    def enable(self):
        self.config = deluge.configmanager.ConfigManager(
            'wslimit.conf', DEFAULT_PREFS)
        self.config.save()
        if platform.system() == 'Windows': component.get("EventManager").register_event_handler("SessionStartedEvent", self.z)

    def z(self):
        memory_limit = self.config['mem'] * 1024 * 1024
        if self.config['prio'] == "normal": 
            prio = 5
        elif self.config['prio'] == "below-normal":
            prio = 4
        elif self.config['prio'] == "medium":
            prio = 3
        elif self.config['prio'] == "low":
            prio = 2
        elif self.config['prio'] == "very-low":
            prio = 1
        pid = os.getpid()
        hProcess = ctypes.windll.kernel32.OpenProcess(0xFFFF, 0, pid)
        ctypes.windll.kernel32.SetProcessWorkingSetSizeEx(hProcess, 67108864, memory_limit, 0x00000004)
        class MEMORY_PRIORITY_INFORMATION(ctypes.Structure):
            _fields_ = [
                ("MemoryPriority", ctypes.c_int)
            ]
        mpi = MEMORY_PRIORITY_INFORMATION(prio)
        size = ctypes.sizeof(mpi)
        ctypes.windll.kernel32.SetProcessInformation(hProcess, 0, ctypes.byref(mpi), size)
        pid = threading.get_ident()
        hProcess = ctypes.windll.kernel32.OpenThread(0xFFFF, 0, pid)
        mpi = MEMORY_PRIORITY_INFORMATION(5)
        size = ctypes.sizeof(mpi)
        ctypes.windll.kernel32.SetThreadInformation(hProcess, 0, ctypes.byref(mpi), size)
        ctypes.windll.kernel32.SetThreadPriority(hProcess, 1)

    def disable(self):
        if platform.system() != 'Windows': return
        component.get("EventManager").deregister_event_handler("SessionStartedEvent", self.z)
        pid = os.getpid()
        hProcess = ctypes.windll.kernel32.OpenProcess(0xFFFF, 0, pid)
        ctypes.windll.kernel32.SetProcessWorkingSetSizeEx(hProcess, 204800, 1413120, 0x00000008)
        class MEMORY_PRIORITY_INFORMATION(ctypes.Structure):
            _fields_ = [
                ("MemoryPriority", ctypes.c_int)
            ]
        mpi = MEMORY_PRIORITY_INFORMATION(5)
        size = ctypes.sizeof(mpi)
        ctypes.windll.kernel32.SetProcessInformation(hProcess, 0, ctypes.byref(mpi), size)
        pid = threading.get_ident()
        hProcess = ctypes.windll.kernel32.OpenThread(0xFFFF, 0, pid)
        ctypes.windll.kernel32.SetThreadPriority(hProcess, 0)

    def update(self):
        pass

    @export
    def set_config(self, config):
        """Sets the config dictionary"""
        for key in config:
            self.config[key] = config[key]
        self.config.save()
        self.z()

    @export
    def get_config(self):
        """Returns the config dictionary"""
        return self.config.config
