/**
 * Script: wslimit.js
 *     The client-side javascript code for the wslimit plugin.
 *
 * Copyright:
 *     (C) mhertz 2022 <yourname@example.com>
 *
 *     This file is part of wslimit and is licensed under GNU GPL 3.0, or
 *     later, with the additional special exception to link portions of this
 *     program with the OpenSSL library. See LICENSE for more details.
 */

wslimitPanel = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        config = Ext.apply({
            border: false,
            title: _("wslimit"),
            autoHeight: true,
        }, config);
        wslimitPanel.superclass.constructor.call(this, config);
    },
    initComponent: function() {
        wslimitPanel.superclass.initComponent.call(this);
        this.opts = new Deluge.OptionsManager();

        var fieldset = this.add({
            xtype: 'fieldset',
            title: _('wslimit settings'),
            autoHeight: true,
            autoWidth: true,
            defaultType: 'textfield'
        });
        this.opts.bind('mem', fieldset.add({
			fieldLabel: _('Memory (MiB) limit'),
            anchor:'100%',
            name: 'mem',
            autoWidth: true,
            value: ''
        }));

        deluge.preferences.on('show', this.onPreferencesShow, this);
    },

    onPreferencesShow: function () {
        deluge.client.wslimit.get_config({
            success: function (config) {
            if (!Ext.isEmpty(config['mem'])) {
                config['mem'] = config['mem'];
            }
           
            //Ext.getCmp("mycheck").checked;
            this.opts.set(config);
            },
            scope: this,
        });
    },
    onApply: function(e) {
        var changed = this.opts.getDirty();
        if (!Ext.isObjectEmpty(changed)) {
            if (!Ext.isEmpty(changed['mem'])) {
            changed['mem'] = changed['mem'];
            }
           
            deluge.client.wslimit.set_config(changed, {
                success: this.onSetConfig,
                scope: this,
            });
        }
    },
    onSetConfig: function() {
        this.opts.commit();
    },
});


wslimitPlugin = Ext.extend(Deluge.Plugin, {
    name: "wslimit",

    onDisable: function() {
        deluge.preferences.removePage(this.prefsPage);
    },

    onEnable: function() {
        this.prefsPage = new wslimitPanel();
        this.prefsPage = deluge.preferences.addPage(this.prefsPage);
    }
});

Deluge.registerPlugin('wslimit', wslimitPlugin);

