# -*- coding: utf-8 -*-
# Copyright (C) 2023 mhertz <yourname@example.com>
#
# Basic plugin template created by the Deluge Team.
#
# This file is part of wslimit and is licensed under GNU GPL 3.0, or later,
# with the additional special exception to link portions of this program with
# the OpenSSL library. See LICENSE for more details.
from __future__ import unicode_literals

import logging

from gi.repository import Gtk

import deluge.component as component
from deluge.plugins.pluginbase import Gtk3PluginBase
from deluge.ui.client import client

from .common import get_resource

log = logging.getLogger(__name__)


class Gtk3UI(Gtk3PluginBase):
    def enable(self):
        self.builder = Gtk.Builder()
        self.builder.add_from_file(get_resource('config.ui'))

        component.get('Preferences').add_page(
            'wslimit', self.builder.get_object('prefs_box'))
        component.get('PluginManager').register_hook(
            'on_apply_prefs', self.on_apply_prefs)
        component.get('PluginManager').register_hook(
            'on_show_prefs', self.on_show_prefs)

        self.sel_func_store = Gtk.ListStore(str)
        self.sel_func_store.append(["normal"])
        self.sel_func_store.append(["below-normal"])
        self.sel_func_store.append(["medium"])
        self.sel_func_store.append(["low"])
        self.sel_func_store.append(["very-low"])
        cell = Gtk.CellRendererText()
        cbo_sel_func = self.builder.get_object("cbo_sel_func")
        cbo_sel_func.pack_start(cell, True)
        cbo_sel_func.add_attribute(cell, 'text', 0)
        cbo_sel_func.set_model(self.sel_func_store)
        cbo_sel_func.set_active(1)
    def disable(self):
        component.get('Preferences').remove_page('wslimit')
        component.get('PluginManager').deregister_hook(
            'on_apply_prefs', self.on_apply_prefs)
        component.get('PluginManager').deregister_hook(
            'on_show_prefs', self.on_show_prefs)

    def on_apply_prefs(self):
        log.debug('applying prefs for wslimit')
        o = self.builder.get_object("cbo_sel_func")
        config = {
            'mem': self.builder.get_object('txt_test').get_value_as_int(),
            'prio': o.get_model()[o.get_active_iter()][0]
        }
        client.wslimit.set_config(config)

    def on_show_prefs(self):
        client.wslimit.get_config().addCallback(self.cb_get_config)

    def cb_get_config(self, config):
        """callback for on show_prefs"""
        self.builder.get_object('txt_test').set_value(config['mem'])

        selected = config['prio']

        for i, row in enumerate(self.sel_func_store):

            if row[0] == selected:
                self.builder.get_object("cbo_sel_func").set_active(i)
                break
        else:
            self.builder.get_object("cbo_sel_func").set_active(0)
